<?php

namespace xenarabia\ThreadsStamp\Admin\Controller;

use xenarabia\ThreadsStamp\Entity\StampsEntity;
use XF\Mvc\FormAction;
use XF\Mvc\ParameterBag;
use XF\Mvc\Reply\View;
use XF\Admin\Controller\AbstractController;


class Stamps extends AbstractController{


    public function actionIndex(ParameterBag $params)
    {
        
        $stamps = $this->finder('xenarabia\ThreadsStamp:StampsEntity')->fetch();

        $page = $this->filterPage($params->page);
        $perPage = 20;
        $stamps = $this->getAllstamps($page, $perPage);
        $viewParams = [
            'stamps' => $stamps->fetch(),
            'page'     => $page,
            'perPage' => $perPage,
            'total'        => $stamps->total()
        ];
        return $this->view('xenarabia\ThreadsStamp\XF:ThreadsStamp\list', 'stamps_list', $viewParams);
    }

    public function actionAdd()
    {
        return $this->view('xenarabia\ThreadsStamp\XF:ThreadsStamp', 'stamp_addEdit', []);
    }

    public function actionEdit(ParameterBag $params)
    {
        $stamp = $this->assertStampsExists($params->s_id);
        $viewParams = [
            'stamp' => $stamp,
        ];

        return $this->view('xenarabia\ThreadsStamp\XF:ThreadsStamp', 'stamp_addEdit', $viewParams);
    }

    public function actionSave(ParameterBag $params)
    {
        $this->assertPostOnly();

        if ($params->s_id) {
            $stamp = $this->assertStampsExists($params->s_id);
            $form = $this->pageSaveProcess($stamp);
        } else {
            $stamp = $this->em()->create('xenarabia\ThreadsStamp:StampsEntity');
            $form = $this->pageSaveProcess($stamp);
        }

        if (get_class($form) === 'XF\Mvc\Reply\Error') {
            return $form;
        } else {
            $form->run();
        }

        return $this->redirect($this->buildLink('threads-stamp'), \XF::phrase('ts_stamp_has_been_saved'));
    }


    protected function pageSaveProcess(StampsEntity $stampsEntity)
    {

        $form = $this->formAction();

        $g_name = $this->filter('s_name', 'str');
        $form->validate(function (FormAction $form) use ($g_name) {
            $length_g_name = strlen($g_name);
            if ($g_name === '' || empty($g_name) || $length_g_name === 0) {
                $form->logError(\XF::phrase('please_enter_stamp_name'), 's_name');
            }
        });


        $upload = $this->request->getFile('stampImage', false);
        if(!$upload)
        {
            return $this->error(\XF::phrase('please_choose_stamp_image'));
        }
        $newFileName = \XF::$time . "-ThreadsStamp." . $upload->getExtension();
        $dataFile = 'data://ThreadsStamp/' . $newFileName;
        \XF\Util\File::copyFileToAbstractedPath($upload->getTempFile(), $dataFile);

        $entityInput = $this->filter([
            's_name'    => 'str'
        ]);
        $entityInput['s_image'] = $newFileName;

        $form->validateEntity($stampsEntity)->basicEntitySave($stampsEntity, $entityInput);

        return $form;
    }



    public function actionDeleteImage(ParameterBag $params)
    {
        $imgStamp = $this->filter('imageName', 'str');
        $uploadFolder = \XF::getRootDirectory() . "/data/ThreadsStamp/" . $imgStamp;

        $deleteImage = file_exists($uploadFolder) ? @unlink($uploadFolder) : null;
        if ($deleteImage) {
            $result['success'] = \XF::phrase('ts_stamp_image_has_been_removed');
        } else {
            $result['error'] = \XF::phrase('ts_stamp_image_has_been_not_removed');
        }
        $view = $this->view('xenarabia\ThreadsStamp\XF:ThreadsStamp', '', []);
		$view->setJsonParam('imgStamp', $result);
        return $view;
    }


    public function actionDelete(ParameterBag $params)
    {

        $stamp = $this->assertStampsExists($params->s_id);

        /** @var \XF\ControllerPlugin\Delete $plugin */
        $plugin = $this->plugin('XF:Delete');
        return $plugin->actionDelete(
            $stamp,
            $this->buildLink('threads-stamp/delete', $stamp),
            $this->buildLink('threads-stamp/edit', $stamp),
            $this->buildLink('threads-stamp'),
            $stamp->s_name
        );

    }

    protected function assertStampsExists($id, $with = null, $phraseKey = null)
    {
        return $this->assertRecordExists('xenarabia\ThreadsStamp:StampsEntity', $id, $with, $phraseKey);
    }

    protected function getAllStamps($page, $perPage)
    {
        $repo = $this->finder('xenarabia\ThreadsStamp:StampsEntity');
        $finder = $repo->order('s_id', 'ASC')
        ->limitByPage($page, $perPage);
        return $finder;
    }
}